import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JFrame;

public class Tetris_004 implements KeyListener{
	JFrame f;//メインウィンドウ
	Canvas canvas;//描画領域
	Image image;//バッファ領域（ダブルバッファリング用）

	Graphics fg;//描画領域用描画オブジェクト（グラフィックスコンテキスト）
	Graphics bg;//ダブルバッファリング用描画オブジェクト（グラフィックスコンテキスト）

	final int FIELD_ROWS=21;//フィールド幅
	final int FIELD_COLS=12;//フィールド高さ
	final int CELL_SIZE=28;//正方形（セル）のサイズ

	int[][] field=new int[FIELD_ROWS][FIELD_COLS];//フィールド用配列
	int[][] block;//ブロック用配列
	int x;//ブロックのx座標
	int y;//ブロックのy座標

	int screenW;//ウィンドウの幅
	int screenH;//ウィンドウの高さ

	//画面の初期化メソッド
	public void initScreen() {
		//画面サイズの設定
		screenW=FIELD_COLS*CELL_SIZE;
		screenH=FIELD_ROWS*CELL_SIZE;

		//フレームの設定
		f=new JFrame();
		f.setTitle("Tetris");
		f.setSize(screenW,screenH);
		f.setResizable(false);//ウィンドウサイズを変更不可に設定
		f.addKeyListener(this);
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setVisible(true);

		//描画領域の設定
		canvas=new Canvas();
		canvas.setSize(screenW,screenH);
		f.add(canvas);

		//描画領域のサイズ調整
        canvas.setPreferredSize(new Dimension(screenW,screenH));
		f.pack();

		//フレームに描画する準備
		image=canvas.createImage(screenW, screenH);
		fg=canvas.getGraphics();
		bg=image.getGraphics();
	}

	//フィールドの初期化メソッド
	public void initField() {
		for(int row=0;row<FIELD_ROWS;row++) {
			for(int col=0;col<FIELD_COLS;col++) {
				if(row==FIELD_ROWS-1||col==0||col==FIELD_COLS-1) {
					field[row][col]=1;
				}else {
					field[row][col]=0;
				}
			}
		}
	}

	//ブロックの初期化メソッド
	public void initBlock() {
		//テスト用にTブロックを設定
		block= new int[][]{
						{0,1,0},
						{1,1,1},
						{0,0,0}
						};

		//フィールド内のブロックの座標の指定
		x=FIELD_COLS/2-1;
		y=0;
	}

	//フィールドの描画メソッド
	public void drawField() {
		for(int row=0;row<FIELD_ROWS;row++) {
			for(int col=0;col<FIELD_COLS;col++) {
				if(field[row][col]!=0) {
					//色をlightGrayに設定
					bg.setColor(Color.lightGray);
					//壁・床の描画
					bg.fillRect(col*CELL_SIZE, row*CELL_SIZE, CELL_SIZE-1, CELL_SIZE-1);
				}else {
					//色をblackに設定
					bg.setColor(Color.black);
					//壁・床の描画
					bg.fillRect(col*CELL_SIZE, row*CELL_SIZE, CELL_SIZE-1, CELL_SIZE-1);
				}
			}
		}
	}

	//ブロックの描画メソッド
	public void drawBlock() {
		for(int row=0;row<block.length;row++) {
			for(int col=0;col<block[row].length;col++) {
				if(block[row][col]!=0) {
					//色をredに設定
					bg.setColor(Color.red);
					//壁・床の描画
					bg.fillRect((col+x)*CELL_SIZE, (row+y)*CELL_SIZE, CELL_SIZE-1, CELL_SIZE-1);
				}
			}
		}
	}


	public Tetris_004(){
		//画面の初期化
		initScreen();

		//フィールド配列の初期化
		initField();

		//ブロック配列の初期化
		initBlock();

		while(true) {
			//描画領域をクリア
			bg.clearRect(0, 0, screenW, screenH);
			//色をlightGrayに設定
			bg.setColor(Color.darkGray);
			//背景の描画
			bg.fillRect(0,0,screenW,screenH);
			//フィールドの描画
			drawField();
			//ブロックの描画
			drawBlock();
			//バッファ領域のイメージを描画領域へ転送
			fg.drawImage(image, 0, 0, canvas);
			try {
				Thread.sleep(200);
			} catch (InterruptedException e) {
				// TODO 自動生成された catch ブロック
				e.printStackTrace();
			}
		}
	}

	public static void main(String[] args){
		new Tetris_004();
	}

	public void keyTyped(KeyEvent e) {

	}

	public void keyPressed(KeyEvent e) {
		switch(e.getKeyCode()) {
		case KeyEvent.VK_UP://ブロック上移動
			y--;
			break;
		case KeyEvent.VK_DOWN://ブロック下移動
			y++;
			break;
		case KeyEvent.VK_LEFT://ブロック左移動
			x--;
			break;
		case KeyEvent.VK_RIGHT://ブロック右移動
			x++;
			break;
		}

		int[][] w=new int[block.length][block.length];
		switch(e.getKeyChar()) {
		case 'z':
		case 'Z':
			//左回転
			for(int row=0;row<block.length;row++) {
				for(int col=0;col<block[row].length;col++) {
					w[row][col]=block[col][block.length-1-row];
				}
			}
			for(int row=0;row<block.length;row++) {
				for(int col=0;col<block[row].length;col++) {
					block[row][col]=w[row][col];
				}
			}
			break;
		case 'x':
		case 'X':
			//右回転
			for(int row=0;row<block.length;row++) {
				for(int col=0;col<block[row].length;col++) {
					w[row][col]=block[block.length-1-col][row];
				}
			}
			for(int row=0;row<block.length;row++) {
				for(int col=0;col<block[row].length;col++) {
					block[row][col]=w[row][col];
				}
			}
			break;
		}
	}

	public void keyReleased(KeyEvent e) {

	}
}
